#include "common/software_version.h"

#include <locale>

#include <boost/format.hpp>

namespace
{
    TypeDeviceNameValue g_typeDeviceNameValue;
}

/**
 * @class TypeDeviceNameValue
 * @brief Verwalten des Gerätekürzels einer Firmwareversion.
 *
 * Diese Klasse mappt den Gerätekürzel der an jeder Firmwareversion angefügt ist gegen eine Nummer.
 */

/**
 * @brief Liefert zur anzugebenden Gerätenummer den entsprechenden Gerätekürzel.
 *
 * @param[in] type Gerätenummer.
 *
 * @return Liefert den zugehörigen Gerätekürzel.
 */
std::string TypeDeviceNameValue::get_device_name( enum_software_version_type type )
{
    std::string res;
    for( auto iter = m_type_device_name_value.begin(); iter != m_type_device_name_value.end(); ++iter )
    {
        if ( type == iter->second.second )
        {
            res = iter->second.first;
            break;
        }
    }

    return res;
}

/**
 * @brief Liefert zum anzugebenden Gerätekürzel die entsprechende Gerätenummer.
 *
 * @param[in] type Gerätekürzel.
 *
 * @return Liefert die zugehörige Gerätenummer.
 */
enum_software_version_type TypeDeviceNameValue::get_device_value( const std::string& str )
{
    auto res = svt_unknown;
    if ( m_type_device_name_value.count( boost::to_lower_copy( str ) ) )
    {
        res = m_type_device_name_value[boost::to_lower_copy( str )].second;
    }

    return res;
}

/**
 * @class SoftwareVersion
 * @brief Verwaltet eine Softwareversion.
 *
 * Verwaltet eine Softwareversion im Format "AA.BB.CC.DD" und arbeitet bei Vergleichen mit den ersten drei Stellen, 
 * die Buildnummer "DD" wird nicht mitverglichen.
 */

/**
 * @class SoftwarebuildVersion
 * @brief Verwaltet eine Softwareversion.
 *
 * Verwaltet eine Softwareversion im Format "AA.BB.CC.DD" und arbeitet bei Vergleichen mit allen Stellen.
 */

SoftwareVersion::SoftwareVersion()
{
    reset();
}

SoftwareVersion::SoftwareVersion( const std::string& version )
{
    reset();
    if ( parse( version ) == false )
    {
        reset();
    }
}

SoftwareVersion::SoftwareVersion( const char* version )
{
    reset();
    if ( parse( version ) == false )
    {
        reset();
    }
}

SoftwareVersion::SoftwareVersion( unsigned int version )
{
    reset();
    if ( parse( version ) == false )
    {
        reset();
    }
}

SoftwareVersion::SoftwareVersion( unsigned char master, char major, unsigned char minor, unsigned char build, enum_software_version_type type_device )
{
    std::string typeDeviceText;
    reset();

    if ( master > 99 )
    {
        master = 99;
    }

    m_master = master;

    if ( major > 99 )
    {
        major = 99;
    }

    m_major = major;

    if ( minor > 99 )
    {
        minor = 99;
    }

    m_minor = minor;

    if ( build > 99 )
    {
        build = 99;
    }

    m_build = build;

    if ( type_device != svt_unknown )
    {
        typeDeviceText = g_typeDeviceNameValue.get_device_name( type_device );
    }

    m_type_device = type_device;
    if ( type_device == svt_unknown )
    {
        m_version = boost::str( boost::format( "%02d.%02d.%02d.%02d" ) 
            % static_cast<int>( m_master ) % static_cast<int>( m_major ) % static_cast<int>( m_minor ) % static_cast<int>( m_build ) );
    }
    else
    {
        m_version = boost::str( boost::format( "%02d.%02d.%02d.%02d.%s" ) 
            % static_cast<int>( m_master ) % static_cast<int>( m_major ) % static_cast<int>( m_minor ) % static_cast<int>( m_build ) % typeDeviceText );
    }
}

SoftwareVersion::~SoftwareVersion()
{
}

void SoftwareVersion::reset()
{
    m_master = 0;
    m_major = 0;
    m_minor = 0;
    m_build = 0;
    m_type_device = svt_unknown;
    m_version = boost::str( boost::format( "%02d.%02d.%02d.%02d" ) 
        % static_cast<int>( m_master ) % static_cast<int>( m_major ) % static_cast<int>( m_minor ) % static_cast<int>( m_build  ) );
}

bool SoftwareVersion::parse( const std::string& version )
{
    // Formatvorlage kurz: XX.XX.XX???
    // Formatvorlage lang: XX.XX.XX.XX.YYYYYYY
    auto not_digit = []( char c ){ return (std::isdigit( c, std::locale() ) == false); };

    if ( version.size() < 8 
        || not_digit( version[0] ) || not_digit( version[1] ) || version[2] != '.' 
        || not_digit( version[3] ) || not_digit( version[4] ) || version[5] != '.'
        || not_digit( version[6] ) || not_digit( version[7] ) 
        || (version.size() > 8 && (version[8] != '.' || not_digit( version[9] ) || not_digit( version[10] ))) )
    {
        // Formatfehler
        return false;
    }

    // Kopie des Versionsstrings halten.
    m_version = version;

    int i;
    // Alle Werte übernehmen
    i = 0;
    m_master = (version[i] - '0') * 10 + (version[i + 1] - '0');

    i += 3;
    m_major  = (version[i] - '0') * 10 + (version[i + 1] - '0');

    i += 3;
    m_minor  = (version[i] - '0') * 10 + (version[i + 1] - '0');

    m_build = 0;
    if ( version.size() > 8 )
    {
        i += 3;
        m_build  = (version[i] - '0') * 10 + (version[i + 1] - '0');
    }

    m_type_device = svt_unknown;
    // Ermittlung des Geraetetyps?
    if ( version.size() > 12 )
    {
        m_type_device = g_typeDeviceNameValue.get_device_value( &version[12] );
    }

    return true;
}

bool SoftwareVersion::parse( unsigned int version )
{
    if ( (((version >> 24) & 0xFF) > 99)
        || (((version >> 16) & 0xFF) > 99)
        || (((version >>  8) & 0xFF) > 99)
        || ((version & 0xFF) > 99) )
    {
        // Formatfehler
        return false;
    }

    // Wert in Member uebernehmen
    m_master = (version >> 24) & 0xFF;
    m_major  = (version >> 16) & 0xFF;
    m_minor  = (version >>  8) & 0xFF;
    m_build  = version & 0xFF;
    m_type_device = svt_unknown;

    m_version = boost::str( boost::format( "%02d.%02d.%02d.%02d" ) 
        % static_cast<int>( m_master ) % static_cast<int>( m_major ) % static_cast<int>( m_minor ) % static_cast<int>( m_build ) );

    return true;
}
