/**
* @file  file_touch_keys.cpp
* @brief Implementierung der Klasse FileTouchKeys.
*/
//////////////////////////////////////////////////////////////////////

#include "file/file_touch_keys.h"

FileTouchKeys::FileTouchKeys()
{
    // Der Standardmäßig zu verarbeitende Datensatztyp
    setRecordType( Record_type::touch_key );

    // Initialisieren
    resetData();

    // Fehlerobjekt zuruecksetzen
    resetError();

}

FileTouchKeys::~FileTouchKeys()
{
    // Speicher freigeben.
    resetData();
}

void FileTouchKeys::resetData()
{
    // Basis Reset
    FileIntelHex::resetData();

    // Member initialisieren
    m_pTouchKeysNode = nullptr;
    for ( auto &key : m_pTouchKeys )
    {
        key = nullptr;
    }
}

void FileTouchKeys::init( unsigned char count, unsigned char deviceNumber )
{
    // Speicher reservieren
    m_buffer.resize( sizeof( TOUCHKEYNODE ) + count * sizeof( unsigned int ) + count * sizeof( TOUCHKEY ) );

    // Hilfszeiger auf Struktur
    m_pTouchKeysNode = (TOUCHKEYNODE *)&m_buffer[0];

    // 1.
    // Zeichensaetze init
    // Grundstruktur aufsetzen
    m_pTouchKeysNode->version = 1;
    m_pTouchKeysNode->cnt     = count;
    m_pTouchKeysNode->temp[0] = deviceNumber;
    m_pTouchKeysNode->temp[1] = 0;
    m_pTouchKeysNode->ptrTouchkeyArray = sizeof( TOUCHKEYNODE );

    // Zeigerarray und Touchkeysstrukturzeiger initialisieren
    std::size_t i = m_pTouchKeysNode->cnt;
    for ( std::size_t j = 0; j < i; ++j )
    {
        // Strukturoffset auf Touchkeystruktur
        *(unsigned int *)&m_buffer[m_pTouchKeysNode->ptrTouchkeyArray + j * sizeof( unsigned int )] = 
            m_pTouchKeysNode->ptrTouchkeyArray + i * sizeof( unsigned int ) + j * sizeof( TOUCHKEY );

        m_pTouchKeys[j] = (TOUCHKEY *)&m_buffer[m_pTouchKeysNode->ptrTouchkeyArray + i * sizeof( unsigned int ) + j * sizeof( TOUCHKEY )];
    }
}

void FileTouchKeys::SetTouchKey( int idx, const TOUCHKEY &tk )
{
    *m_pTouchKeys[idx] = tk;
}

TOUCHKEY FileTouchKeys::GetTouchKey( int idx )
{
    return *m_pTouchKeys[idx];
}

bool FileTouchKeys::load( const std::string &filePath )
{
    // Lesen der Hexdatei
    if ( FileIntelHex::load( filePath ) )
    {
        // Member durch gelesene Daten initialisieren und pruefen
        if ( m_buffer.size() )
        {
            // Hilfszeiger auf Struktur
            m_pTouchKeysNode = (TOUCHKEYNODE *)&m_buffer[0];

            // 1.
            // Touchkeys pruefen

            // Zeigerarray und Touchkeystrukturzeiger initialisieren
            std::size_t i = m_pTouchKeysNode->cnt;
            if ( i > max_touch_keys )
            {
                // Fehler zuweisen
                setErrorState( Error_state::format );
                setErrorDetail( boost::str( boost::format( "%1%(%2%): %3%" ) % __FILE__ % __LINE__ % errorText() ) );
            }
            else
            {
                for ( std::size_t j = 0; j < i; ++j )
                {
                    m_pTouchKeys[j] = (TOUCHKEY *)&m_buffer[m_pTouchKeysNode->ptrTouchkeyArray + i * sizeof( unsigned int ) + j * sizeof( TOUCHKEY )];
                }
            }
        }
        else
        {
            // Fehler zuweisen
            setErrorState( Error_state::no_data );
            setErrorDetail( boost::str( boost::format( "%1%(%2%): %3%" ) % __FILE__ % __LINE__ % errorText() ) );
        }
    }

    return (errorState() == Error_state::none);
}



bool FileTouchKeys::save( const std::string &filePath, bool appendToFile /*= false*/, bool appendEndLine /*= true */ )
{
    return FileIntelHex::save( filePath, appendToFile, appendEndLine );
}
